@@ -18,6 +18,14 @@
 ** LINUX-port:  Februar 1999
 **
 ** $Log: fetchdef.c,v $
+** Revision 1.22  2004/07/08 12:52:03  gerlof
+** Adapted for kernel version 2.6.
+** Support of new disk-names.
+**
+** Revision 1.21  2002/09/03 12:06:16  gerlof
+** Corrected number of open files and maximum number of open inodes.
+** 
+**
 ** Revision 1.20  2001/03/16 09:37:18  root
 ** Reimplement FTP/HTTP counters.
 **
@@ -92,7 +100,9 @@
 ** See the GNU General Public License for more details.
 */
 
-static char rcsid[] = "$Id: fetchdef.c,v 1.20 2001/03/16 09:37:18 root Exp $";
+static const char rcsid[] = "$Id: fetchdef.c,v 1.22 2004/07/08 12:52:03 gerlof Exp $";
+
+static int	isrealdisk(char *, char *, int);
 
 #define	EQ	0
 
@@ -110,7 +120,7 @@ genget(struct fetchdef *fdef, struct osrel *osrel, struct genstat *gs)
 	register int	nr, i;
 	FILE 		*fp;
 	char		linebuf[8192], nam[64];
-	unsigned long	cnts[MAXCNT];
+	count_t		cnts[MAXCNT];
 
 	memset(gs, 0, sizeof(struct genstat));
 
@@ -123,11 +133,16 @@ genget(struct fetchdef *fdef, struct osrel *osrel, struct genstat *gs)
 		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
 		{
 		     nr = sscanf(linebuf,
-			"%s  %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
-			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
-			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
-			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
-			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu\n",
+			"%s   %lld %lld %lld %lld %lld %lld "
+			"%lld %lld %lld %lld %lld %lld "
+			"%lld %lld %lld %lld %lld %lld %lld "
+			"%lld %lld %lld %lld %lld %lld "
+			"%lld %lld %lld %lld %lld %lld %lld "
+			"%lld %lld %lld %lld %lld %lld "
+			"%lld %lld %lld %lld %lld %lld %lld "
+			"%lld %lld %lld %lld %lld %lld "
+			"%lld %lld %lld %lld %lld %lld %lld "
+			"%lld %lld %lld %lld %lld %lld\n",
 				nam,
 				&cnts[0],  &cnts[1],  &cnts[2],  &cnts[3],
 				&cnts[4],  &cnts[5],  &cnts[6],  &cnts[7],
@@ -156,6 +171,13 @@ genget(struct fetchdef *fdef, struct osrel *osrel, struct genstat *gs)
 				gs->cpu_system	= cnts[2];
 				gs->cpu_idle	= cnts[3];
 
+				if (osrel->rel >= 2 && osrel->vers >= 6)
+				{
+					gs->cpu_wait	= cnts[4];
+					gs->cpu_irq	= cnts[5];
+					gs->cpu_sirq	= cnts[6];
+				}
+
 				continue;
 			}
 
@@ -169,6 +191,14 @@ genget(struct fetchdef *fdef, struct osrel *osrel, struct genstat *gs)
 					gs->per_cpu_nice[cpunr]	   = cnts[1];
 					gs->per_cpu_system[cpunr]  = cnts[2];
 					gs->per_cpu_idle[cpunr]	   = cnts[3];
+
+					if (osrel->rel  >= 2 &&
+					    osrel->vers >= 6)
+					{
+					    gs->per_cpu_wait[cpunr] = cnts[4];
+					    gs->per_cpu_irq [cpunr] = cnts[5];
+					    gs->per_cpu_sirq[cpunr] = cnts[6];
+					}
 				}
 
 				continue;
@@ -196,7 +226,6 @@ genget(struct fetchdef *fdef, struct osrel *osrel, struct genstat *gs)
 			{
 				gs->pgpgin	= cnts[0];
 				gs->pgpgout	= cnts[1];
-
 				continue;
 			}
 
@@ -204,19 +233,18 @@ genget(struct fetchdef *fdef, struct osrel *osrel, struct genstat *gs)
 			{
 				gs->pswpin	= cnts[0];
 				gs->pswpout	= cnts[1];
-
 				continue;
 			}
 
 			/*
-			** handle disk-statistics kernel-version >= 2.4
+			** handle disk-statistics
+			** only for kernel-version 2.4
 			*/
 			if ( memcmp("disk_io", nam, 7) == EQ)
 			{
-				char 		*p;
-				unsigned long	major, minor, tot,
-						rio, rblk, wio, wblk;
-				int		dkix;
+				char 	*p;
+				count_t	major, minor, tot, rio, rblk, wio, wblk;
+				int	dkix;
 
 				/*
 				** redo the parsing due to format:
@@ -232,15 +260,16 @@ genget(struct fetchdef *fdef, struct osrel *osrel, struct genstat *gs)
 				dkix = 0;
 
 				while ((nr=sscanf(p,
-					   "(%lu,%lu):(%lu,%lu,%lu,%lu,%lu) ",
+					   "(%lld,%lld):(%lld,%lld,%lld,"
+					   "%lld,%lld) ",
 						&major, &minor, &tot,
 						&rio, &rblk, &wio, &wblk)) ==7)
 				{ 
-					gs->dk_drive_maj[dkix]	= major;
-					gs->dk_drive_min[dkix]	= minor;
-					gs->dk_drive_tot[dkix]	= tot;
-					gs->dk_drive_rio[dkix]	= rio;
-					gs->dk_drive_wio[dkix]	= wio;
+					gs->dk_drive_maj [dkix]	= major;
+					gs->dk_drive_min [dkix]	= minor;
+					gs->dk_drive_tot [dkix]	= tot;
+					gs->dk_drive_rio [dkix]	= rio;
+					gs->dk_drive_wio [dkix]	= wio;
 					gs->dk_drive_rblk[dkix]	= rblk;
 					gs->dk_drive_wblk[dkix]	= wblk;
 
@@ -332,11 +361,16 @@ genget(struct fetchdef *fdef, struct osrel *osrel, struct genstat *gs)
 		    	int irq, max;
 
 		    	nr = sscanf(linebuf,
-			"%d: %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
-			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
-			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
-			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
-			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu\n",
+			            "%d:  %lld %lld %lld %lld %lld %lld %lld "
+			            "%lld %lld %lld %lld %lld "
+			            "%lld %lld %lld %lld %lld %lld %lld %lld "
+			            "%lld %lld %lld %lld %lld "
+			            "%lld %lld %lld %lld %lld %lld %lld %lld "
+			            "%lld %lld %lld %lld %lld "
+			            "%lld %lld %lld %lld %lld %lld %lld %lld "
+			            "%lld %lld %lld %lld %lld "
+			            "%lld %lld %lld %lld %lld %lld %lld %lld "
+			            "%lld %lld %lld %lld %lld\n",
 				&irq,
 				&cnts[0],  &cnts[1],  &cnts[2],  &cnts[3],
 				&cnts[4],  &cnts[5],  &cnts[6],  &cnts[7],
@@ -374,7 +408,7 @@ genget(struct fetchdef *fdef, struct osrel *osrel, struct genstat *gs)
 	{
 		if ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
 		{
-		    	nr = sscanf(linebuf, "%f %f %f %lu/%lu",
+		    	nr = sscanf(linebuf, "%f %f %f %lld/%lld",
 				&(gs->loadavg1),
 				&(gs->loadavg5),
 				&(gs->loadavg15),
@@ -384,6 +418,50 @@ genget(struct fetchdef *fdef, struct osrel *osrel, struct genstat *gs)
 
 		fclose(fp);
 	}
+
+	/*
+	** gather page-/swap-counters kernel version >= 2.6
+	*/
+	if ( (fp = fopen("/proc/vmstat", "r")) != NULL)
+	{
+		int	nrfields = 4;
+		count_t	cnt;
+
+		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL &&
+								nrfields > 0)
+		{
+			nr = sscanf(linebuf, "%s %lld", nam, &cnt);
+
+			if (nr < 2)		/* invalid line --> skip */
+				continue;
+
+			if ( strcmp("pgpgin", nam) == EQ)
+			{
+                                gs->pgpgin = cnt;
+				nrfields--;
+			}
+
+			if ( strcmp("pgpgout", nam) == EQ)
+			{
+                                gs->pgpgout = cnt;
+				nrfields--;
+			}
+
+			if ( strcmp("pswpin", nam) == EQ)
+			{
+                                gs->pswpin = cnt;
+				nrfields--;
+			}
+
+			if ( strcmp("pswpout", nam) == EQ)
+			{
+                                gs->pswpout = cnt;
+				nrfields--;
+			}
+		}
+
+		fclose(fp);
+	}
 }
 
 /*
@@ -392,46 +470,64 @@ genget(struct fetchdef *fdef, struct osrel *osrel, struct genstat *gs)
 void
 memget(struct fetchdef *fdef, struct osrel *osrel, struct memstat *mi)
 {
-	register int	nr, nrfills=2;
+	register int	nr, nrfields=7;
 	FILE 		*fp;
 	char		linebuf[1024];
 	char		itsname[32];
-	unsigned long	cnts[10];
+	count_t		cnt;
 
 	memset(mi, 0, sizeof(struct memstat));
 
 	if ( (fp = fopen("/proc/meminfo", "r")) == NULL)
 		return;
 
-	while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+	while ( fgets(linebuf, sizeof(linebuf), fp) != NULL && nrfields > 0)
 	{
-		nr = sscanf(linebuf,
-			"%s %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu\n",
-			itsname,
-			&cnts[0],  &cnts[1],  &cnts[2],  &cnts[3],  &cnts[4],
-			&cnts[5],  &cnts[6],  &cnts[7],  &cnts[8],  &cnts[9]);
+		nr = sscanf(linebuf, "%s %lld", itsname, &cnt);
 
-		if (nr < 2)		/* headerline ? --> skip */
+		if (nr < 2)		/* invalid line --> skip */
 			continue;
 
-		if ( strcmp("Mem:", itsname) == EQ)
+		if ( strcmp("MemTotal:", itsname) == EQ)
 		{
-			memcpy(mi->memory, cnts, sizeof(mi->memory));
-
-			if (--nrfills > 0)
-				continue;
-			else
-				break;
+			mi->memtot = cnt;
+			nrfields--;
 		}
 
-		if ( strcmp("Swap:", itsname) == EQ)
+		if ( strcmp("MemFree:", itsname) == EQ)
 		{
-			memcpy(mi->swap, cnts, sizeof(mi->swap));
+			mi->memfree = cnt;
+			nrfields--;
+		}
 
-			if (--nrfills > 0)
-				continue;
-			else
-				break;
+		if ( strcmp("Buffers:", itsname) == EQ)
+		{
+			mi->membuf = cnt;
+			nrfields--;
+		}
+
+		if ( strcmp("Cached:", itsname) == EQ)
+		{
+			mi->memcache = cnt;
+			nrfields--;
+		}
+
+		if ( strcmp("Slab:", itsname) == EQ)
+		{
+			mi->memslab = cnt;
+			nrfields--;
+		}
+
+		if ( strcmp("SwapTotal:", itsname) == EQ)
+		{
+			mi->swptot = cnt;
+			nrfields--;
+		}
+
+		if ( strcmp("SwapFree:", itsname) == EQ)
+		{
+			mi->swpfree = cnt;
+			nrfields--;
 		}
 	}
 
@@ -442,6 +538,8 @@ memget(struct fetchdef *fdef, struct osrel *osrel, struct memstat *mi)
 /*
 ** Gather network-interface statistics from /proc and store in binary form.
 */
+int		getifstats(struct osrel *, struct ifstat *, int);
+
 void
 ifinit(struct fetchdef *fdef, struct osrel *osrel)
 {
@@ -475,7 +573,6 @@ int
 getifstats(struct osrel *osrel, struct ifstat *ifsp, int maxif)
 {
 	register int	i=0, nr;
-	int		notused;
 	FILE 		*fp;
 	char		linebuf[1024], *cp;
 
@@ -496,8 +593,8 @@ getifstats(struct osrel *osrel, struct ifstat *ifsp, int maxif)
 			memset(ifsp, 0, sizeof(struct ifstat));
 
 	                nr = sscanf(linebuf,
-				    "%s  %lu %lu %lu %lu %lu %lu %lu "
-				    "%lu %lu %lu %lu\n",
+				    "%s   %lld %lld %lld %lld %lld %lld %lld "
+				    "%lld %lld %lld %lld\n",
                                           ifsp->name,
                                         &(ifsp->rpack),
                                         &(ifsp->rerrs),
@@ -520,8 +617,9 @@ getifstats(struct osrel *osrel, struct ifstat *ifsp, int maxif)
 		else
 		{
 			nr = sscanf(linebuf,
-			 	    "%s %lu %lu %lu %lu %lu %lu %lu %lu %lu "
-				    "%lu %lu %lu %lu %lu %lu %lu\n",
+			 	    "%s   %lld %lld %lld %lld %lld %lld %lld "
+			            "%lld %lld %lld %lld %lld %lld %lld %lld "
+			            "%lld\n",
 					  ifsp->name,
 					&(ifsp->rbyte),
 					&(ifsp->rpack),
@@ -565,72 +663,72 @@ struct icmpv6_mib	icmpv6_tmp;
 struct udp_mib		udpv6_tmp;
 
 struct v6tab {
-	char 		*nam;
-	unsigned long	*val;
+	char 	*nam;
+	count_t *val;
 } v6tab[] = {
-    "Ip6InReceives",		&ipv6_tmp.Ip6InReceives,
-    "Ip6InHdrErrors",		&ipv6_tmp.Ip6InHdrErrors,
-    "Ip6InTooBigErrors",	&ipv6_tmp.Ip6InTooBigErrors,
-    "Ip6InNoRoutes",		&ipv6_tmp.Ip6InNoRoutes,
-    "Ip6InAddrErrors",		&ipv6_tmp.Ip6InAddrErrors,
-    "Ip6InUnknownProtos",	&ipv6_tmp.Ip6InUnknownProtos,
-    "Ip6InTruncatedPkts",	&ipv6_tmp.Ip6InTruncatedPkts,
-    "Ip6InDiscards",		&ipv6_tmp.Ip6InDiscards,
-    "Ip6InDelivers",		&ipv6_tmp.Ip6InDelivers,
-    "Ip6OutForwDatagrams",	&ipv6_tmp.Ip6OutForwDatagrams,
-    "Ip6OutRequests",		&ipv6_tmp.Ip6OutRequests,
-    "Ip6OutDiscards",		&ipv6_tmp.Ip6OutDiscards,
-    "Ip6OutNoRoutes",		&ipv6_tmp.Ip6OutNoRoutes,
-    "Ip6ReasmTimeout",		&ipv6_tmp.Ip6ReasmTimeout,
-    "Ip6ReasmReqds",		&ipv6_tmp.Ip6ReasmReqds,
-    "Ip6ReasmOKs",		&ipv6_tmp.Ip6ReasmOKs,
-    "Ip6ReasmFails",		&ipv6_tmp.Ip6ReasmFails,
-    "Ip6FragOKs",		&ipv6_tmp.Ip6FragOKs,
-    "Ip6FragFails",		&ipv6_tmp.Ip6FragFails,
-    "Ip6FragCreates",		&ipv6_tmp.Ip6FragCreates,
-    "Ip6InMcastPkts",		&ipv6_tmp.Ip6InMcastPkts,
-    "Ip6OutMcastPkts",		&ipv6_tmp.Ip6OutMcastPkts,
+    {"Ip6InReceives",		&ipv6_tmp.Ip6InReceives,                     },
+    {"Ip6InHdrErrors",		&ipv6_tmp.Ip6InHdrErrors,                    },
+    {"Ip6InTooBigErrors",	&ipv6_tmp.Ip6InTooBigErrors,                 },
+    {"Ip6InNoRoutes",		&ipv6_tmp.Ip6InNoRoutes,                     },
+    {"Ip6InAddrErrors",		&ipv6_tmp.Ip6InAddrErrors,                   },
+    {"Ip6InUnknownProtos",	&ipv6_tmp.Ip6InUnknownProtos,                },
+    {"Ip6InTruncatedPkts",	&ipv6_tmp.Ip6InTruncatedPkts,                },
+    {"Ip6InDiscards",		&ipv6_tmp.Ip6InDiscards,                     },
+    {"Ip6InDelivers",		&ipv6_tmp.Ip6InDelivers,                     },
+    {"Ip6OutForwDatagrams",	&ipv6_tmp.Ip6OutForwDatagrams,               },
+    {"Ip6OutRequests",		&ipv6_tmp.Ip6OutRequests,                    },
+    {"Ip6OutDiscards",		&ipv6_tmp.Ip6OutDiscards,                    },
+    {"Ip6OutNoRoutes",		&ipv6_tmp.Ip6OutNoRoutes,                    },
+    {"Ip6ReasmTimeout",		&ipv6_tmp.Ip6ReasmTimeout,                   },
+    {"Ip6ReasmReqds",		&ipv6_tmp.Ip6ReasmReqds,                     },
+    {"Ip6ReasmOKs",		&ipv6_tmp.Ip6ReasmOKs,                       },
+    {"Ip6ReasmFails",		&ipv6_tmp.Ip6ReasmFails,                     },
+    {"Ip6FragOKs",		&ipv6_tmp.Ip6FragOKs,                        },
+    {"Ip6FragFails",		&ipv6_tmp.Ip6FragFails,                      },
+    {"Ip6FragCreates",		&ipv6_tmp.Ip6FragCreates,                    },
+    {"Ip6InMcastPkts",		&ipv6_tmp.Ip6InMcastPkts,                    },
+    {"Ip6OutMcastPkts",		&ipv6_tmp.Ip6OutMcastPkts,                   },
     
-    "Icmp6InMsgs",		&icmpv6_tmp.Icmp6InMsgs,
-    "Icmp6InErrors",		&icmpv6_tmp.Icmp6InErrors,
-    "Icmp6InDestUnreachs",	&icmpv6_tmp.Icmp6InDestUnreachs,
-    "Icmp6InPktTooBigs",	&icmpv6_tmp.Icmp6InPktTooBigs,
-    "Icmp6InTimeExcds",		&icmpv6_tmp.Icmp6InTimeExcds,
-    "Icmp6InParmProblems",	&icmpv6_tmp.Icmp6InParmProblems,
-    "Icmp6InEchos",		&icmpv6_tmp.Icmp6InEchos,
-    "Icmp6InEchoReplies",	&icmpv6_tmp.Icmp6InEchoReplies,
-    "Icmp6InGroupMembQueries",	&icmpv6_tmp.Icmp6InGroupMembQueries,
-    "Icmp6InGroupMembResponses",
-				&icmpv6_tmp.Icmp6InGroupMembResponses,
-    "Icmp6InGroupMembReductions",
-				&icmpv6_tmp.Icmp6InGroupMembReductions,
-    "Icmp6InRouterSolicits",	&icmpv6_tmp.Icmp6InRouterSolicits,
-    "Icmp6InRouterAdvertisements",
-				&icmpv6_tmp.Icmp6InRouterAdvertisements,
-    "Icmp6InNeighborSolicits",	&icmpv6_tmp.Icmp6InNeighborSolicits,
-    "Icmp6InNeighborAdvertisements",
-				&icmpv6_tmp.Icmp6InNeighborAdvertisements,
-    "Icmp6InRedirects",		&icmpv6_tmp.Icmp6InRedirects,
-    "Icmp6OutMsgs",		&icmpv6_tmp.Icmp6OutMsgs,
-    "Icmp6OutDestUnreachs",	&icmpv6_tmp.Icmp6OutDestUnreachs,
-    "Icmp6OutPktTooBigs",	&icmpv6_tmp.Icmp6OutPktTooBigs,
-    "Icmp6OutTimeExcds",	&icmpv6_tmp.Icmp6OutTimeExcds,
-    "Icmp6OutParmProblems",	&icmpv6_tmp.Icmp6OutParmProblems,
-    "Icmp6OutEchoReplies",	&icmpv6_tmp.Icmp6OutEchoReplies,
-    "Icmp6OutRouterSolicits",	&icmpv6_tmp.Icmp6OutRouterSolicits,
-    "Icmp6OutNeighborSolicits",	&icmpv6_tmp.Icmp6OutNeighborSolicits,
-    "Icmp6OutNeighborAdvertisements",
-				&icmpv6_tmp.Icmp6OutNeighborAdvertisements,
-    "Icmp6OutRedirects",	&icmpv6_tmp.Icmp6OutRedirects,
-    "Icmp6OutGroupMembResponses",
-				&icmpv6_tmp.Icmp6OutGroupMembResponses,
-    "Icmp6OutGroupMembReductions",
-				&icmpv6_tmp.Icmp6OutGroupMembReductions,
+    {"Icmp6InMsgs",		&icmpv6_tmp.Icmp6InMsgs,                     },
+    {"Icmp6InErrors",		&icmpv6_tmp.Icmp6InErrors,                   },
+    {"Icmp6InDestUnreachs",	&icmpv6_tmp.Icmp6InDestUnreachs,             },
+    {"Icmp6InPktTooBigs",	&icmpv6_tmp.Icmp6InPktTooBigs,               },
+    {"Icmp6InTimeExcds",	&icmpv6_tmp.Icmp6InTimeExcds,                },
+    {"Icmp6InParmProblems",	&icmpv6_tmp.Icmp6InParmProblems,             },
+    {"Icmp6InEchos",		&icmpv6_tmp.Icmp6InEchos,                    },
+    {"Icmp6InEchoReplies",	&icmpv6_tmp.Icmp6InEchoReplies,              },
+    {"Icmp6InGroupMembQueries",	&icmpv6_tmp.Icmp6InGroupMembQueries,         },
+    {"Icmp6InGroupMembResponses",
+				&icmpv6_tmp.Icmp6InGroupMembResponses,       },
+    {"Icmp6InGroupMembReductions",
+				&icmpv6_tmp.Icmp6InGroupMembReductions,      },
+    {"Icmp6InRouterSolicits",	&icmpv6_tmp.Icmp6InRouterSolicits,           },
+    {"Icmp6InRouterAdvertisements",
+				&icmpv6_tmp.Icmp6InRouterAdvertisements,     },
+    {"Icmp6InNeighborSolicits",	&icmpv6_tmp.Icmp6InNeighborSolicits,         },
+    {"Icmp6InNeighborAdvertisements",
+				&icmpv6_tmp.Icmp6InNeighborAdvertisements,   },
+    {"Icmp6InRedirects",	&icmpv6_tmp.Icmp6InRedirects,                },
+    {"Icmp6OutMsgs",		&icmpv6_tmp.Icmp6OutMsgs,                    },
+    {"Icmp6OutDestUnreachs",	&icmpv6_tmp.Icmp6OutDestUnreachs,            },
+    {"Icmp6OutPktTooBigs",	&icmpv6_tmp.Icmp6OutPktTooBigs,              },
+    {"Icmp6OutTimeExcds",	&icmpv6_tmp.Icmp6OutTimeExcds,               },
+    {"Icmp6OutParmProblems",	&icmpv6_tmp.Icmp6OutParmProblems,            },
+    {"Icmp6OutEchoReplies",	&icmpv6_tmp.Icmp6OutEchoReplies,             },
+    {"Icmp6OutRouterSolicits",	&icmpv6_tmp.Icmp6OutRouterSolicits,          },
+    {"Icmp6OutNeighborSolicits",&icmpv6_tmp.Icmp6OutNeighborSolicits,        },
+    {"Icmp6OutNeighborAdvertisements",
+				&icmpv6_tmp.Icmp6OutNeighborAdvertisements,  },
+    {"Icmp6OutRedirects",	&icmpv6_tmp.Icmp6OutRedirects,               },
+    {"Icmp6OutGroupMembResponses",
+				&icmpv6_tmp.Icmp6OutGroupMembResponses,      },
+    {"Icmp6OutGroupMembReductions",
+				&icmpv6_tmp.Icmp6OutGroupMembReductions,     },
 
-    "Udp6InDatagrams",		&udpv6_tmp.UdpInDatagrams,
-    "Udp6NoPorts",		&udpv6_tmp.UdpNoPorts,
-    "Udp6InErrors",		&udpv6_tmp.UdpInErrors,
-    "Udp6OutDatagrams",		&udpv6_tmp.UdpOutDatagrams,
+    {"Udp6InDatagrams",		&udpv6_tmp.UdpInDatagrams,                   },
+    {"Udp6NoPorts",		&udpv6_tmp.UdpNoPorts,                       },
+    {"Udp6InErrors",		&udpv6_tmp.UdpInErrors,                      },
+    {"Udp6OutDatagrams",	&udpv6_tmp.UdpOutDatagrams,                  },
 };
 
 int	v6tab_entries = sizeof(v6tab)/sizeof(struct v6tab);
@@ -642,7 +740,7 @@ netget(struct fetchdef *fdef, struct osrel *osrel, struct netstat *ns)
 	FILE 		*fp;
 	char		linebuf[1024];
 	char		itsname[32];
-	unsigned long	cnts[40];
+	count_t		cnts[40];
 
 	memset(ns, 0, sizeof(struct netstat));
 
@@ -654,10 +752,11 @@ netget(struct fetchdef *fdef, struct osrel *osrel, struct netstat *ns)
 		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
 		{
 		    nr = sscanf(linebuf,
-			"%s  %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
-			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
-			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
-			"%lu %lu\n",
+			        "%s   %lld %lld %lld %lld %lld %lld %lld %lld "
+			        "%lld %lld %lld %lld %lld %lld %lld %lld %lld "
+			        "%lld %lld %lld %lld %lld %lld %lld %lld %lld "
+			        "%lld %lld %lld %lld %lld %lld %lld %lld %lld "
+				"%lld %lld %lld %lld %lld\n",
 				itsname,
 				&cnts[0],  &cnts[1],  &cnts[2],  &cnts[3],
 				&cnts[4],  &cnts[5],  &cnts[6],  &cnts[7],
@@ -715,7 +814,7 @@ netget(struct fetchdef *fdef, struct osrel *osrel, struct netstat *ns)
 
 		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
 		{
-		   	nr = sscanf(linebuf, "%s %lu", itsname, &cnts[0]);
+		   	nr = sscanf(linebuf, "%s %lld", itsname, &cnts[0]);
 
 		   	if (strcmp(v6tab[cur].nam, itsname) == EQ)
 		   	{
@@ -755,8 +854,8 @@ nfsget(struct fetchdef *fdef, struct osrel *osrel, struct nfsstat *ns)
 	FILE 		*fp;
 	char		linebuf[1024];
 	char		itsname[32];
-	unsigned long	cnts[32];
-	void		nfsfill(char *, unsigned long [], int,
+	count_t		cnts[32];
+	void		nfsfill(char *, count_t [], int,
 					struct nfsstat *, int);
 
 	memset(ns, 0, sizeof(struct nfsstat));
@@ -769,10 +868,10 @@ nfsget(struct fetchdef *fdef, struct osrel *osrel, struct nfsstat *ns)
 		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
 		{
 			nr = sscanf(linebuf,
-				"%s  %lu %lu %lu %lu %lu %lu %lu %lu %lu "
-				"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
-				"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
-				"%lu %lu %lu\n",
+				"%s   %lld %lld %lld %lld %lld %lld %lld %lld "
+				"%lld %lld %lld %lld %lld %lld %lld %lld %lld "
+				"%lld %lld %lld %lld %lld %lld %lld %lld %lld "
+				"%lld %lld %lld %lld %lld %lld\n",
 				   itsname,
 				   &cnts[0],  &cnts[1],  &cnts[2],  &cnts[3],
 				   &cnts[4],  &cnts[5],  &cnts[6],  &cnts[7],
@@ -801,10 +900,10 @@ nfsget(struct fetchdef *fdef, struct osrel *osrel, struct nfsstat *ns)
 		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
 		{
 			nr = sscanf(linebuf,
-				"%s  %lu %lu %lu %lu %lu %lu %lu %lu %lu "
-				"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
-				"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
-				"%lu %lu %lu\n",
+				"%s   %lld %lld %lld %lld %lld %lld %lld %lld "
+				"%lld %lld %lld %lld %lld %lld %lld %lld %lld "
+				"%lld %lld %lld %lld %lld %lld %lld %lld %lld "
+				"%lld %lld %lld %lld %lld %lld\n",
 				   itsname,
 				   &cnts[0],  &cnts[1],  &cnts[2],  &cnts[3],
 				   &cnts[4],  &cnts[5],  &cnts[6],  &cnts[7],
@@ -826,7 +925,7 @@ nfsget(struct fetchdef *fdef, struct osrel *osrel, struct nfsstat *ns)
 }
 
 void
-nfsfill(char *nam, unsigned long cnts[], int nc,
+nfsfill(char *nam, count_t cnts[], int nc,
 			struct nfsstat *ns, int isclient)
 {
 	if ( strcmp("net", nam) == EQ)
@@ -970,11 +1069,10 @@ nfsfill(char *nam, unsigned long cnts[], int nc,
 void
 tabget(struct fetchdef *fdef, struct osrel *osrel, struct tabstat *ps)
 {
-	register int	nr, baselen;
+	register int	baselen;
 	FILE 		*fp;
 	char		linebuf[1024], firstword[80], basepath[132];
-	long		cnt;
-	unsigned long	dummy;
+	count_t		cnt;
 
 	memset(ps, 0, sizeof(struct tabstat));
 
@@ -985,7 +1083,7 @@ tabget(struct fetchdef *fdef, struct osrel *osrel, struct tabstat *ps)
 	{
 		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
 		{
-			sscanf(linebuf, "%s %lu", firstword, &cnt);
+			sscanf(linebuf, "%s %lld", firstword, &cnt);
 	
 			if ( strcmp(firstword, "processes") == EQ)
 			{
@@ -1017,13 +1115,13 @@ tabget(struct fetchdef *fdef, struct osrel *osrel, struct tabstat *ps)
 	*/
 	if ( (fp = fopen("/proc/sys/fs/dquot-max", "r")) != NULL)
 	{
-		fscanf(fp, "%lu", &(ps->maxdquot));
+		fscanf(fp, "%lld", &(ps->maxdquot));
 		fclose(fp);
 	}
 
 	if ( (fp = fopen("/proc/sys/fs/dquot-nr", "r")) != NULL)
 	{
-		fscanf(fp, "%lu", &(ps->curdquot));
+		fscanf(fp, "%lld", &(ps->curdquot));
 		fclose(fp);
 	}
 
@@ -1032,13 +1130,13 @@ tabget(struct fetchdef *fdef, struct osrel *osrel, struct tabstat *ps)
 	*/
 	if ( (fp = fopen("/proc/sys/fs/super-max", "r")) != NULL)
 	{
-		fscanf(fp, "%lu", &(ps->maxsuper));
+		fscanf(fp, "%lld", &(ps->maxsuper));
 		fclose(fp);
 	}
 
 	if ( (fp = fopen("/proc/sys/fs/super-nr", "r")) != NULL)
 	{
-		fscanf(fp, "%lu", &(ps->cursuper));
+		fscanf(fp, "%lld", &(ps->cursuper));
 		fclose(fp);
 	}
 
@@ -1061,7 +1159,7 @@ tabget(struct fetchdef *fdef, struct osrel *osrel, struct tabstat *ps)
 
 	if ( (fp = fopen(basepath, "r")) != NULL)
 	{
-		fscanf(fp, "%lu", &(ps->maxfiles));
+		fscanf(fp, "%lld", &(ps->maxfiles));
 		fclose(fp);
 	}
 
@@ -1069,8 +1167,12 @@ tabget(struct fetchdef *fdef, struct osrel *osrel, struct tabstat *ps)
 
 	if ( (fp = fopen(basepath, "r")) != NULL)
 	{
-		fscanf(fp, "%lu %lu", &dummy, &(ps->curfiles));
+		count_t	alloctot, allocfree;
+
+		fscanf(fp, "%lld %lld", &alloctot, &allocfree);
 		fclose(fp);
+
+		ps->curfiles = alloctot - allocfree;
 	}
 
 	/*
@@ -1080,7 +1182,7 @@ tabget(struct fetchdef *fdef, struct osrel *osrel, struct tabstat *ps)
 
 	if ( (fp = fopen(basepath, "r")) != NULL)
 	{
-		fscanf(fp, "%lu", &(ps->maxinode));
+		fscanf(fp, "%lld", &(ps->maxinode));
 		fclose(fp);
 	}
 
@@ -1088,7 +1190,7 @@ tabget(struct fetchdef *fdef, struct osrel *osrel, struct tabstat *ps)
 
 	if ( (fp = fopen(basepath, "r")) != NULL)
 	{
-		fscanf(fp, "%lu", &(ps->curinode));
+		fscanf(fp, "%lld %lld", &(ps->curinode), &(ps->maxinode));
 		fclose(fp);
 	}
 }
@@ -1097,32 +1199,33 @@ tabget(struct fetchdef *fdef, struct osrel *osrel, struct tabstat *ps)
 ** Gather statistics about the utilization of disk-partitions from /proc and
 ** store in binary form.
 */
+#define	SCANDK	"%255s %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld"
+#define	SCANDS	"%hu %hu "
+#define	SCANPT	"%hu %hu %lld "
+
+static char usediskstats;
+
 void
 dkinit(struct fetchdef *fdef, struct osrel *osrel)
 {
 	int		numpart=0, nr=0;
 	FILE		*fp;
 	struct dkstat	dkstat;
-	char		buf[256];
+	char		buf[256], diskname[256];
 
 	/*
-	** define number of partitions and whether or not partition-stats
-	** are supported on this system;
+	** define number of partitions and define if per-partition stats
+	** are supported on this system
 	*/
-	if ( (fp = fopen("/proc/partitions", "r")) == NULL)
+	if ( (fp = fopen("/proc/partitions", "r")) != NULL)
 	{
-		fdef->cdef.kinst = 0;
-		return;
-	}
-
-	while ( fgets(buf, sizeof(buf), fp) )
-	{
-		nr = sscanf(buf, "%hu %hu %lu %15s %lu %lu %lu %lu %lu "
-				 "%lu %lu %lu %lu  %lu %lu",
+		while ( fgets(buf, sizeof(buf), fp) )
+		{
+			nr = sscanf(buf, SCANPT SCANDK,
                        		&dkstat.major,
 				&dkstat.minor,
                        		&dkstat.totsize,
-                       		 dkstat.name,
+                       		 diskname,
 
                        		&dkstat.readblocks,
                        		&dkstat.readmerges,
@@ -1138,10 +1241,56 @@ dkinit(struct fetchdef *fdef, struct osrel *osrel)
                        		&dkstat.rdwr_msecs,
                        		&dkstat.avq);
 
-		if (nr >= 15)
-			numpart++;
+			if (nr >= 8 &&
+				isrealdisk(diskname, dkstat.name, MAXDKNAME) )
+				numpart++;
+		}
+
+		fclose(fp);
+	}
+
+	if (numpart > 0)	/* partition-stats found */
+	{
+		fdef->cdef.kinst = numpart;
+		return;
+	}
+
+	/*
+	** no per-partition statistics found; try /proc/diskstats (>= 2.6)
+	*/
+	if ( (fp = fopen("/proc/diskstats", "r")) != NULL)
+	{
+		while ( fgets(buf, sizeof(buf), fp) )
+		{
+			nr = sscanf(buf, SCANDS SCANDK,
+                       		&dkstat.major,
+				&dkstat.minor,
+                       		 diskname,
+
+                       		&dkstat.readblocks,
+                       		&dkstat.readmerges,
+                       		&dkstat.readsectors,
+                       		&dkstat.read_msecs,
+
+                       		&dkstat.writblocks,
+                       		&dkstat.writmerges,
+                       		&dkstat.writsectors,
+                       		&dkstat.writ_msecs,
+
+                       		&dkstat.reqpending,
+                       		&dkstat.rdwr_msecs,
+                       		&dkstat.avq);
+
+			if (nr >= 7 &&
+				isrealdisk(diskname, dkstat.name, MAXDKNAME) )
+				numpart++;
+		}
+
+		if (numpart > 0)
+			usediskstats++;
+
+		fclose(fp);
 	}
-	fclose(fp);
 
 	fdef->cdef.kinst = numpart;
 }
@@ -1151,24 +1300,23 @@ dkget(struct fetchdef *fdef, struct osrel *osrel, struct dkstat *ds)
 {
 	FILE	*fp;
 	int	numpart=0, nr=0;
-	char	buf[1024];
+	char	buf[1024], diskname[256];
 
 	if (fdef->cdef.kinst == 0)
 		return;
 
 	memset(ds, 0, sizeof(struct dkstat) * fdef->cdef.kinst);
 
-	if ( (fp = fopen("/proc/partitions", "r")) == NULL)
-		return;
-
-	while ( fgets(buf, sizeof(buf), fp) && numpart < fdef->cdef.kinst)
+	if (usediskstats)
 	{
-		nr = sscanf(buf, "%d  %d  %lu %63s %lu %lu %lu %lu %lu "
-				 "%lu %lu %lu %lu  %lu %lu",
+		fp = fopen("/proc/diskstats",  "r");
+
+		while (fgets(buf, sizeof(buf), fp) && numpart< fdef->cdef.kinst)
+		{
+			nr = sscanf(buf, SCANDS SCANDK,
                       		&(ds->major),
 				&(ds->minor),
-                       		&(ds->totsize),
-                       		  ds->name,
+                       		  diskname,
 
                        		&(ds->readblocks),
                        		&(ds->readmerges),
@@ -1184,16 +1332,145 @@ dkget(struct fetchdef *fdef, struct osrel *osrel, struct dkstat *ds)
                        		&(ds->rdwr_msecs),
                        		&(ds->avq));
 
-		if (nr == 15)
-		{
+			if (nr < 7 ||
+				!isrealdisk(diskname, ds->name, MAXDKNAME) )
+				continue;
+
+			if (nr == 7) 	/* partition i.s.o. full disk ? */
+			{
+				ds->writblocks  = ds->readsectors;
+				ds->writsectors	= ds->read_msecs;
+				ds->readsectors	= ds->readmerges;
+				ds->readmerges	= 0;
+				ds->read_msecs	= 0;
+				ds->writmerges	= 0;
+				ds->writ_msecs	= 0;
+				ds->reqpending	= 0;
+				ds->rdwr_msecs	= 0;
+				ds->avq		= 0;
+			}
+
 			numpart++;
 			ds++;
 		}
 	}
+	else
+	{
+		fp = fopen("/proc/partitions", "r");
+
+		while (fgets(buf, sizeof(buf), fp) && numpart< fdef->cdef.kinst)
+		{
+			nr = sscanf(buf, SCANPT SCANDK,
+                      		&(ds->major),
+				&(ds->minor),
+                       		&(ds->totsize),
+                       		  diskname,
+                       		&(ds->readblocks),
+                       		&(ds->readmerges),
+                       		&(ds->readsectors),
+                       		&(ds->read_msecs),
+
+                       		&(ds->writblocks),
+                       		&(ds->writmerges),
+                       		&(ds->writsectors),
+                       		&(ds->writ_msecs),
+
+                       		&(ds->reqpending),
+                       		&(ds->rdwr_msecs),
+                       		&(ds->avq));
+
+			if (nr >= 8 && isrealdisk(diskname, ds->name,MAXDKNAME))
+			{
+				numpart++;
+				ds++;
+			}
+		}
+	}
 
 	fclose(fp);
 }
 
+/*
+** set of subroutines to determine which disks should be monitored
+** and to translate long name strings into short name strings
+*/
+static void
+nullmodname(char *curname, char *newname, int maxlen)
+{
+	strncpy(newname, curname, maxlen-1);
+	*(newname+maxlen-1) = 0;
+}
+
+static void
+abbrevname1(char *curname, char *newname, int maxlen)
+{
+	char	cutype[128];
+	int	hostnum, busnum, targetnum, lunnum, partnum, nr;
+
+	nr = sscanf(curname, "%[^/]/host%d/bus%d/target%d/lun%d/part%d",
+			cutype, &hostnum, &busnum, &targetnum,
+			        &lunnum,  &partnum);
+
+	if (nr == 5)	/* full disk */
+		snprintf(newname, maxlen, "%c-h%db%dt%d", 
+			cutype[0], hostnum, busnum, targetnum);
+	else		/* partition */
+		snprintf(newname, maxlen, "%c-h%db%dt%d.%d", 
+			cutype[0], hostnum, busnum, targetnum, partnum);
+}
+
+/*
+** table contains the names (in regexp format) of valid disks
+** to be supported, together a function to modify the name-strings
+** (i.e. to abbreviate long strings);
+** this table is used in the function isrealdisk()
+*/
+static struct {
+	char 	*regexp;
+	regex_t	compreg;
+	void	(*modname)(char *, char *, int);
+} validdisk[] = {
+	{	"^sd[a-z][a-z]*",			{0}, 	nullmodname, },
+	{	"^hd[a-z]",				{0},	nullmodname, },
+	{	"^rd/c[0-9][0-9]*d[0-9][0-9]*",		{0},	nullmodname, },
+	{	"^cciss/c[0-9][0-9]*d[0-9][0-9]*",	{0},	nullmodname, },
+	{ 	"/host.*/bus.*/target.*/lun",		{0},	abbrevname1, },
+};
+
+static int
+isrealdisk(char *curname, char *newname, int maxlen)
+{
+	static int	firstcall = 1;
+	register int	i;
+
+	if (firstcall)		/* compile the regular expressions */
+	{
+		for (i=0; i < sizeof validdisk/sizeof validdisk[0]; i++)
+			regcomp(&validdisk[i].compreg, validdisk[i].regexp,
+								REG_NOSUB);
+		firstcall = 0;
+	}
+
+	/*
+	** try to recognize one of the compiled regular expressions
+	*/
+	for (i=0; i < sizeof validdisk/sizeof validdisk[0]; i++)
+	{
+		if (regexec(&validdisk[i].compreg, curname, 0, NULL, 0) == 0)
+		{
+			/*
+			** name-string recognized; modify name-string
+			*/
+			(*validdisk[i].modname)(curname, newname, maxlen);
+
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
+
 /*
 ** Gather statistics about the utilization of sockets from /proc and
 ** store in binary form.
@@ -1201,10 +1478,10 @@ dkget(struct fetchdef *fdef, struct osrel *osrel, struct dkstat *ds)
 void
 sockget(struct fetchdef *fdef, struct osrel *osrel, struct sockstat *ss)
 {
-	FILE 		*fp;
-	char		linebuf[1024];
-	char		itsname[32], notused[64];
-	unsigned long	nowuse, maxuse;
+	FILE 	*fp;
+	char	linebuf[1024];
+	char	itsname[32], notused[64];
+	count_t	nowuse, maxuse;
 
 	memset(ss, 0, sizeof(struct sockstat));
 
@@ -1214,7 +1491,7 @@ sockget(struct fetchdef *fdef, struct osrel *osrel, struct sockstat *ss)
 		{
 			maxuse = 0;	/* 2.4 kernel: no support max usage */
 
-			if ( sscanf(linebuf, "%s  %s %lu %s %lu\n",
+			if ( sscanf(linebuf, "%s  %s %lld %s %lld\n",
 				itsname, notused, &nowuse, notused, &maxuse)<3)
 					continue;
 
@@ -1242,7 +1519,7 @@ sockget(struct fetchdef *fdef, struct osrel *osrel, struct sockstat *ss)
 		{
 			maxuse = 0;	/* 2.4 kernel: no support max usage */
 
-			if ( sscanf(linebuf, "%s  %s %lu %s %lu\n",
+			if ( sscanf(linebuf, "%s  %s %lld %s %lld\n",
 				itsname, notused, &nowuse, notused, &maxuse)<3)
 					continue;
 
@@ -1402,7 +1679,7 @@ ftpget(struct fetchdef *fdef, struct osrel *osrel, struct ftpstat *fs)
 	FILE 		*fp;
 	char		linebuf[1024], filename[256], symname[256];
 	int			i;
-	unsigned long	notused;
+	count_t		notused;
 
 	memset(fs, 0, sizeof(struct ftpstat) * fdef->cdef.kinst);
 
@@ -1414,7 +1691,7 @@ ftpget(struct fetchdef *fdef, struct osrel *osrel, struct ftpstat *fs)
 		if  ( fgets(linebuf, sizeof(linebuf), fp) == NULL)
 			break;
 
-		sscanf(linebuf, "%lf %lf %lf %lf %lf %lf %lu %lu %lu %s %s\n",
+		sscanf(linebuf, "%lf %lf %lf %lf %lf %lf %lld %lld %lld %s %s\n",
 			&fs->ocumxfer, &fs->ocumseconds, &fs->ocumkbytes,
 			&fs->icumxfer, &fs->icumseconds, &fs->icumkbytes,
 			&notused, &notused, &notused, filename, symname);
@@ -1451,7 +1728,7 @@ httpget(struct fetchdef *fdef, struct osrel *osrel, struct httpstat *hs)
 	register int	i;
 	FILE 		*fp;
 	char		linebuf[1024], filename[256], symname[256];
-	unsigned long	notused;
+	count_t		notused;
 
 	memset(hs, 0, sizeof(struct httpstat) * fdef->cdef.kinst);
 
@@ -1463,7 +1740,7 @@ httpget(struct fetchdef *fdef, struct osrel *osrel, struct httpstat *hs)
 		if  ( fgets(linebuf, sizeof(linebuf), fp) == NULL)
 			break;
 
-		sscanf(linebuf, "%lf %lf %lf %lf %lf %lu %lu %lu %s %s\n",
+		sscanf(linebuf, "%lf %lf %lf %lf %lf %lld %lld %lld %s %s\n",
 			&hs->cumhead, &hs->cumgets, &hs->cumputs,
 			&hs->cumpost, &hs->cumdels,
 			&notused, &notused, &notused, filename, symname);
@@ -1536,19 +1813,19 @@ int	oncecnt = sizeof(oncedef)/sizeof(struct fetchdef);
 	/*******************************************************/
 struct fetchdef sampdef[] =
 {
-	"genstat",	sizeof(struct genstat),		 1, 0,       genget,
-	"memstat",	sizeof(struct memstat),		 1, 0,       memget,
-	"ifstat",	sizeof(struct ifstat),		 1, ifinit,  ifget,
-	"netstat",	sizeof(struct netstat),		 1, 0,       netget,
-	"nfsstat",	sizeof(struct nfsstat),		 1, 0,       nfsget,
-	"tabstat",	sizeof(struct tabstat),		 1, 0,       tabget,
-	"partstat",	sizeof(struct dkstat),		 1, dkinit,  dkget,
-	"sockstat",	sizeof(struct sockstat),	 1, 0,       sockget,
-	"ttystat",	sizeof(struct serialstat), 	32, 0,       ttyget,
+	{ {"genstat",   sizeof(struct genstat),	    1}, 0,       genget,  },
+	{ {"memstat",   sizeof(struct memstat),	    1}, 0,       memget,  },
+	{ {"ifstat",    sizeof(struct ifstat),	    1}, ifinit,  ifget,   },
+	{ {"netstat",   sizeof(struct netstat),	    1}, 0,       netget,  },
+	{ {"nfsstat",   sizeof(struct nfsstat),	    1}, 0,       nfsget,  },
+	{ {"tabstat",   sizeof(struct tabstat),	    1}, 0,       tabget,  },
+	{ {"partstat",  sizeof(struct dkstat),	    1}, dkinit,  dkget,   },
+	{ {"sockstat",  sizeof(struct sockstat),    1}, 0,       sockget, },
+	{ {"ttystat",   sizeof(struct serialstat), 32}, 0,       ttyget,  },
  
 #ifdef	WEBSUPPORT
-	"ftpstat",	sizeof(struct ftpstat),		 1, ftpinit,  ftpget,
-	"httpstat",	sizeof(struct httpstat),	 1, httpinit, httpget,
+	{ {"ftpstat",   sizeof(struct ftpstat),	    1}, ftpinit,  ftpget, },
+	{ {"httpstat",  sizeof(struct httpstat),    1}, httpinit, httpget,},
 #endif
 };
 
