@@ -16,6 +16,9 @@
 ** LINUX-port:  Februar 1999
 ** 
 ** $Log: atsar.c,v $
+** Revision 1.9  2004/07/08 12:52:38  gerlof
+** Adapted for kernel version 2.6.
+**
 ** Revision 1.8  2001/03/14 11:09:44  gerlof
 ** Minor bug-fix.
 **
@@ -53,7 +56,7 @@
 ** See the GNU General Public License for more details.
 */
 
-static char rcsid[] = "$Id: atsar.c,v 1.8 2001/03/14 11:09:44 gerlof Exp $";
+static const char rcsid[] = "$Id: atsar.c,v 1.9 2004/07/08 12:52:38 gerlof Exp $";
 
 #define	EQUAL	0
 #define	SECSDAY	86400
@@ -97,7 +100,7 @@ char		*psamp, *csamp;		/* per-sample counter values     */
 ** function prototypes
 */
 int	readset(struct samphdr *, void *);
-int	readin(int, void *, int);
+int	readin(int, char *, int);
 void	prusage(char *);
 void	printhead(struct utsname *, time_t);
 void	mktim(time_t, char *);
@@ -362,7 +365,7 @@ main(int argc, char *argv[])
 	** first read the general file-header and check if
 	** it really concerns a statistics file
 	*/
-	if ( readin(ifd, &filehdr, sizeof(filehdr)) == 0 )
+	if ( readin(ifd, (char *)&filehdr, sizeof(filehdr)) == 0 )
 		exit(0);
 
 	if (filehdr.magic != ATMAGIC)
@@ -395,7 +398,8 @@ main(int argc, char *argv[])
 		exit(5);
 	}
 
-	if ( readin(ifd, oncelist, filehdr.oncecnt*sizeof(struct countdef))==0)
+	if ( readin(ifd, (char *)oncelist,
+	            filehdr.oncecnt*sizeof(struct countdef))==0)
 		exit(0);
 
 	/*
@@ -407,7 +411,7 @@ main(int argc, char *argv[])
 		exit(5);
 	}
 
-	if ( readin(ifd, oncenames, filehdr.oncenames) == 0)
+	if ( readin(ifd, (char *)oncenames, filehdr.oncenames) == 0)
 		exit(0);
 
 	/*
@@ -435,7 +439,8 @@ main(int argc, char *argv[])
 		exit(5);
 	}
 
-	if ( readin(ifd, samplist, filehdr.sampcnt*sizeof(struct countdef)) ==0)
+	if ( readin(ifd, (char *)samplist,
+	            filehdr.sampcnt*sizeof(struct countdef)) ==0)
 		exit(0);
 
 	/*
@@ -447,7 +452,7 @@ main(int argc, char *argv[])
 		exit(5);
 	}
 
-	if ( readin(ifd, sampnames, filehdr.sampnames) == 0)
+	if ( readin(ifd, (char *)sampnames, filehdr.sampnames) == 0)
 		exit(0);
 
 	/*
@@ -483,7 +488,7 @@ main(int argc, char *argv[])
 		exit(5);
 	}
 
-	if ( readin(ifd, conce, filehdr.oncevars) == 0)
+	if ( readin(ifd, (char *)conce, filehdr.oncevars) == 0)
 		exit(0);
 
 	/*
@@ -780,7 +785,7 @@ readset(struct samphdr *hdr, void *cnt)
 	/*
 	** read header info
 	*/
-	if ( readin(ifd, hdr, sizeof(struct samphdr)) == 0) 
+	if ( readin(ifd, (char *)hdr, sizeof(struct samphdr)) == 0) 
 		return(0); 				/* EOF */
 
 	if (hdr->cntsize == 0)
@@ -789,7 +794,7 @@ readset(struct samphdr *hdr, void *cnt)
 	/*
 	** read counter contents
 	*/
-	if ( readin(ifd, cnt, hdr->cntsize) == 0)
+	if ( readin(ifd, (char *)cnt, hdr->cntsize) == 0)
 		return(0);				/* EOF */
 
 	return(1);
@@ -807,7 +812,7 @@ readset(struct samphdr *hdr, void *cnt)
 **			1 - Success
 */
 int
-readin(int fd, void *buf, int sz)
+readin(int fd, char *buf, int sz)
 {
 	register int n;
 
@@ -822,8 +827,8 @@ readin(int fd, void *buf, int sz)
 		if (n == 0)
 			return(0);		/* EOF reached */
 
-		sz		-= n;
-		(char *) buf	+= n;
+		sz	-= n;
+		buf	+= n;
 	}
 	return(1);
 }
