@@ -17,7 +17,9 @@
 #include <fcntl.h>
 #include <ctype.h>
 #include <sys/socket.h>
-#include "linuxstat.h"
 #include <string.h>
+#include <regex.h>
+#include <stdlib.h>
 
+#include "linuxstat.h"
 #include "atsar.h"
